"""videoconverter.cli module -- Command Line Interface (CLI).

This module is designed to be used with the cli script (`bin/pyvideoc`).
Its purpose is to provide end user an easy to use command to convert videos.
"""

import concurrent.futures
import glob
import logging
import ntpath
import os
import re

import argparse

from pyvideoconverter.video import Video
from pyvideoconverter.parallel import SubprocessPool


def _get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description='Convert videos from a file format to another.')

    parser.add_argument('-i', '--input', nargs='+', help='input files')
    parser.add_argument('-R', '--recursive', help='dir,extension of inputs')
    parser.add_argument(
        '-o', '--output', help='output filename format (with extension)', required=True)
    parser.add_argument(
        '--remove-source', help='Remove source(s). Use it carefully.', action="store_true")
    parser.add_argument('-s', '--speedup',
                        help='Number of threads to use.', type=int, default=10)
    parser.add_argument('-p', '--preset', help='Compression ratio / encoding speed.', choices=[
        'ultrafast', 'superfast', 'veryfast', 'faster', 'fast',
        'medium', 'slow', 'slower', 'veryslow'
    ], default='ultrafast')
    return parser


def is_f_str(s: str):
    return bool(re.match(r"^f'.*'$", s) or re.match(r'^f".*"$', s))


def get_dir(path: str):
    if os.path.sep not in path:
        return '.'
    return os.path.sep.join(path.split(os.path.sep)[0:-1])


def get_dynamic_argument(argument: str, variables: dict) -> str:
    return eval(argument, dict(
        __builtins__=None,
        **variables
    ))


def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)


class CliConverter:
    def __init__(self) -> None:
        self.log = logging.getLogger(self.__class__.__name__)

    def parse(self):
        _parser = _get_parser()
        self.args = _parser.parse_args()

        if self.args.input and self.args.recursive:
            _parser.error('Cannot use both `--input/-i` and `--recursive`.')

        if self.args.recursive:
            self.get_recursive_inputs()
        else:
            self.inputs = self.args.input

        if not is_f_str(self.args.output):
            self.args.output = "f'%s'" % self.args.output

        self.get_outputs()

    def get_recursive_inputs(self):
        self.inputs = glob.glob(self.args.recursive, recursive=True)

    def get_outputs(self):
        outputs = list()

        for input in self.inputs:
            input_filename = f = path_leaf(input)
            input_rawname = r = input_filename.split('.')[0]
            input_dir = d = get_dir(input)

            output = get_dynamic_argument(self.args.output, {
                'input_filename': input_filename,
                'input_rawname': input_rawname,
                'input_dir': input_dir,
                'f': f, 'r': r, 'd': d
            })

            outputs.append(output)

        self.outputs = outputs

    def proceed(self):
        inputs = self.inputs
        outputs = self.outputs
        commands = list()

        for input, output in zip(inputs, outputs):
            v = Video(input, preset=self.args.preset)
            commands.append(v.get_command(dst=output))

        with SubprocessPool(max_workers=self.args.speedup) as executor:
            futures = list()

            for command in commands:
                print(command)
                futures.append(executor.submit(command))

            for future in concurrent.futures.as_completed(futures):
                future.result()  # TODO: Handle results.

        if self.args.remove_source:
            for file in inputs:
                os.remove(file)

    @property
    def args(self):
        return self._args

    @args.setter
    def args(self, ags):

        if (ags.remove_source):
            self.log.warning('Please use `remove-source` carefully.')

        self._args = ags



from concurrent.futures import ThreadPoolExecutor
from subprocess import Popen, PIPE


class SubprocessPool(ThreadPoolExecutor):
    def _call(self, command):
        process = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
        return process.communicate()[0]

    def submit(self, command):
        return super().submit(self._call, command=command)



import os
import re
from subprocess import (
    PIPE,
    Popen,
)

FFMPEG_ARGS = {
    'vcodec': 'libx265',
    'crf': 28,
}

EXTS_INFO = {
    'mp4': ['mp4', 'm4v'],
    'mov': ['mov', 'qt'],
    'avi': ['avi'],
    'flv': ['flv'],
    'wmv': ['wmv', 'asf'],
    'mpeg': ['mpeg', 'mpg', 'vob'],
    'mkv': ['mkv'],
}


def _prepare(s):
    match = re.match(r'^[a-zA-Z0-9\-\"/\.\'\ \\_]*$', s)

    if not match:
        raise OSError('Invalid argument `%s`' % s)

    return match.group(0)


def get_video_type(path_or_filename):
    _ext = os.path.basename(path_or_filename).split('.')[-1]

    for vtype, file_ext in EXTS_INFO.items():
        if _ext.lower() in file_ext:
            return vtype

    raise VideoError('Unknown file extension `%s`' % _ext)


class VideoError(Exception):
    pass


class Video:
    ext = ''
    ext_info = {}
    command = ''

    def __init__(self, path, preset='ultrafast') -> None:
        self.path = _prepare(path)

        if not self.path:
            raise VideoError(f'Invalid source file {path}.')

        self.ext = get_video_type(self.path)

    def get_command(self, dst: str, **kwargs):
        dest = _prepare(dst)

        if not dest:
            raise VideoError(f'Invalid destination file {dst}.')

        self.command = command = (
            f'ffmpeg -i "{self.path}" -vcodec {FFMPEG_ARGS["vcodec"]} '
            f'-preset {_prepare(kwargs.get("preset", "ultrafast"))} '
            f'-crf {FFMPEG_ARGS["crf"]} "{dest}"'
        )
        return command

    def convert(self, dst: str):
        if not self.command:
            self.get_command(dst)

        process = Popen(self.command, shell=True, stdout=PIPE)

        return process.communicate()[0]



__author__ = 'Félix HERBINET'

import os
import re
from setuptools import setup, find_packages

PKG_NAME = 'pyvideoconverter'
PKG_DESC = 'Video converter developped in Python3.'

with open('requirements.txt') as reqf:
    requirements = reqf.read().splitlines()

_VFILE = os.path.join('src', PKG_NAME, '_version.py')

with open(_VFILE, 'rt') as vf:
    vline = vf.read()
    vregex = r'^__version__ = [\'"]([^\'"]*)[\'"]$'
    match = re.search(vregex, vline, re.M)

    if not match:
        raise RuntimeError('Unable to find version string in %s' % _VFILE)

    version = match.group(1)


setup(
    python_requires='>=3.6',
    name=PKG_NAME, version=version, description=PKG_DESC,
    author=__author__, author_email='fherbine@student.42.fr',
    license='', platforms='any',
    url='https://github.com/fherbine/%s' % PKG_NAME,
    package_dir={'': 'src'}, packages=find_packages('src'),
    include_package_data=True, requires=requirements, setup_requires=['wheel'],
    scripts=[os.path.join('src', PKG_NAME, 'bin', 'pyvideoc')]
)
